;;; win32-script.el
;;; Author: yamagus@kw.netlaputa.or.jp (YAMAGUCHI, Shuhei)
;;; Modified by H.Miyashita.
;;; Version 0.2
;; (require "win32-script")
;; (define-process-argument-editing
;;   "/[^./]+$"
;;   'script-process-argument-editing-function 'last)

(defvar script-process-argument-editing-list
  '(("/sh$" "sh.exe"
     (lambda (x) (general-process-argument-editing-function x 'cygnus t)))
    ("/bash$" "bash.exe"
     (lambda (x) (general-process-argument-editing-function x 'cygnus t)))
    ("/perl$" "perl.exe"
     (lambda (x) (general-process-argument-editing-function x 'msvc t)))
    ("/t?csh$" "tcsh.exe"
     (lambda (x) (general-process-argument-editing-function x 'msvc t)))))

(defvar script-process-pathext '(".com" ".exe" ".bat" ".cmd"))
(defvar script-process-bufsiz 256)
(defvar script-process-buffer-tmp " *exec script*")

(defun win32-openp (command-name)
  "Locate the full path name of external-command COMMAND-NAME."
  (interactive "sExternal-command: ")
  (catch 'tag
    (let (path)
      (if (file-name-absolute-p command-name)
	  (if (and (file-executable-p command-name)
		   (null (file-directory-p command-name)))
	      (throw 'tag command-name)
	    (mapcar
	     (lambda (suf)
	       (setq path (expand-file-name (concat command-name suf)))
	       (and (file-executable-p path)
		    (null (file-directory-p path))
		    (throw 'tag path)))
	     (if (null script-process-pathext)
		 '("")
	       script-process-pathext)))
	(mapcar
	 (lambda (dir)
	   (mapcar
	    (lambda (suf)
	      (setq path (expand-file-name (concat command-name suf) dir))
	      (and (file-executable-p path)
		   (null (file-directory-p path))
		   (throw 'tag path)))
	    (if (null script-process-pathext)
		'("")
	      (append (list "") script-process-pathext))))
	 exec-path))) nil))

(defun script-process-get-command (script-path)
  "Find out external-command that executes the script SCRIPT-PATH.
Return value is a list of arguments, and car of the list is argv[0]."
  (interactive "fScript: ")
  (let ((buf (generate-new-buffer script-process-buffer-tmp))
	limit args)
    (unwind-protect
	(save-excursion
	  (set-buffer buf)
	  (condition-case nil
	      (progn
		(insert-file-contents script-path nil 0 script-process-bufsiz)
		(goto-line 2)
		(setq limit (point))
		(goto-char 1)
		(if (re-search-forward
		     "\\`#![ \t]*\\([^ \t\n]+\\)[ \t]*" limit t)
		    (while
			(progn
			  (setq args
				(nconc args
				       (list
					(buffer-substring (match-beginning 1)
							  (match-end 1)))))
			  (re-search-forward "\\([^ \t\n]+\\)[ \t]*"
					     limit t))))
		args)
	    (file-error nil)))
      (kill-buffer buf))))
	    

(defun script-process-argument-editing-function (argument)
  "Find out external-command that executes the script SCRIPT-PATH.
Return value is a list of arguments, and car of the list is argv[0]."
  (interactive)
  (let (argv0 sargs func)
    (catch 'tag
      (setq func default-process-argument-editing-function)
      (if (setq sargs (script-process-get-command (car argument)))
	  (progn
	    (setq argv0 (car sargs))
	    (mapcar
	     (lambda (pat)
	       (and (string-match (car pat) argv0)
		    (setq argv0 (win32-openp (nth 1 pat)))
		    (setq func (nth 2 pat))
		    (setq argument
			  (append (list argv0) (cdr sargs) argument))
		    (throw 'tag (cons argv0 (funcall func argument)))))
	     script-process-argument-editing-list)
	    (setq argument
		  (append (list argv0) (cdr sargs) argument))
	    (throw 'tag (cons argv0 (funcall func argument)))))
      (funcall func argument))))

(provide 'win32-script)
